# This file contains solution for quizzes
rm(list=ls())
# sessionInfo()
# R version 4.0.2 (2020-06-22)
# Platform: x86_64-w64-mingw32/x64 (64-bit)
# Running under: Windows 10 x64 (build 19042)
# RStudio Version 1.4.1717

if(!require('rgdal')) install.packages('rgdal', dependencies=T); library('rgdal')
# Shapefile loading---------------
setwd("../Topic2 - Spatial Statistics/Quiz")
austria <- readOGR("STATISTIK_AUSTRIA_POLBEZ_20110101.shp",stringsAsFactors=F)
# Loading data---------------
silcAUT <- read.csv("silcAUT.csv", sep=";")

# Exercise 1: 
# 1.	Considering the districts as the areas of interest (variable: BKZ), 
# build a binary weights matrix using Rook'c criterion
if(!require('spdep')) install.packages('spdep', dependencies=T); library('spdep')
neigh<-poly2nb(austria,row.names =austria$NAME,queen=FALSE)
austria.sp_w<-nb2mat(neigh, glist=NULL, style="B", zero.policy=TRUE)
austria.lw <- nb2listw(neigh, style="B", zero.policy=TRUE)


# 2.	Considering the districts as the areas of interest (variable: BKZ), 
# build a row standardised weights matrix using Queen'c criterion
neigh<-poly2nb(austria,row.names =austria$NAME,queen=T)
austria.sp_w<-nb2mat(neigh, glist=NULL, style="W", zero.policy=TRUE)
austria.lw <- nb2listw(neigh, style="W", zero.policy=TRUE)


# 3.	Considering the income as the target variable, calculate and 
# comment the Moran's I using the weights matrix defined at point 2
Silc.Spat <- merge(austria, silcAUT, by.x="ID", by.y="Districts_ID")
Moran.I<-moran.test(x=Silc.Spat$inc,listw=austria.lw,
                    zero.policy=TRUE,  na.action=na.exclude)
Moran.I
# positive and signifcant spatial autocorrelation

# 4.	Plot the Moran's diagram 
moran.plot(Silc.Spat$inc, austria.lw,
           labels=FALSE,
           xlab="Observed distribution of mean income", 
           ylab="Spatially lagged mean income")

# 5.	Test the presence of spatial autocorrelation in the following OLS model:
model1 <- inc ~ female+ eqsize+ unempl_ben + age_ben

mod.ols <- lm(model1, data=Silc.Spat)
lm.morantest(mod.ols,austria.lw, zero.policy = T, 
             alternative="two.sided")

# 6.	What kind of spatial model is more appropriate for modelling the 
# income in Austria? (considering model1)
lm.LMtests(mod.ols,austria.lw,test="LMerr")
lm.LMtests(mod.ols,austria.lw,test="LMlag")
# the p-value of both tests us below the .05 level and this means that
# we need to run the robust version of these tests
lm.LMtests(mod.ols,austria.lw,test="RLMerr")
lm.LMtests(mod.ols,austria.lw,test="RLMlag")
# for robust version of the tests, only the one for the lag model 
# has p-value<0.05 significant: estimating a
# spatial lag model is preferable rather than a spatial error model

# 7.	Apply a test on the "common factor hypothesis" to choose 
# between SEM and SDM
if(!require('spatialreg')) install.packages('spatialreg', dependencies=T); library('spatialreg')
mod.sdm<-lagsarlm(model1, data=Silc.Spat, austria.lw, type="mixed")
mod.sem<-errorsarlm(model1, data=Silc.Spat, austria.lw)
print(LR.sarlm(mod.sdm,mod.sem))
# the common factor hypothesis in the SDM model is not
# rejected at alpha=0.05 (p-value = 0.07)

# 8.	Fit SAR model. Evaluate if it is providing a better fit of OL 
# and evaluate the autoregressive parameter.
mod.sar<-lagsarlm(model1, data=Silc.Spat, austria.lw)
summary(mod.sar)
# no gain with respect OLS in term of AIC
# positive and significant spatial dependence

# 9.	Fit SAR model and estimate the direct, indirect and 
# total effects with the standard errors
impactssdm <- impacts(mod.sar,listw=austria.lw,R=500)
summary(impactssdm, short=TRUE, zstats=TRUE)
